package LLI::Node;

# This is a wrapper for our data.  Nothing touches the store here.
#
# $Id: Node.pm,v 1.1 2003/03/24 01:05:11 mankins Exp $

use LLI::Cell;

use strict;

my $VERSION = 1.00;

sub new 
  {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my ($type) = (@_);
    
    my $self = {};
    bless ($self, $class);

    $self->set_type($type) if ($type); # default to generic

    return $self;
  }

sub get_name
  {
    # This returns what our node name is.

    # The name depends on what type of node this is: presence, generic, or cell

    # presence = a person/object/entity's location that requires stricter subscriptions
    # cell     = a cell-based store for earth-based location information.  Public access.
    # generic  = temporary nodes to provide answers to searches/queries

    my $self = shift;

    if ($self->get_type() eq 'presence')
      {
	# p:matt@foo.com/Watch
	$self->{'name'} = "p:".$self->{'jid'};

	my $resource = $self->get_resource();
	if ($resource)
	  {
	    $self->{'name'} .= "/".$resource;
	  }
      }
    elsif ($self->get_type() eq 'cell')
      {
	# c:10Dlat,10Dlon:1Dlat,1Dlon:Mlat,Mlon:Slat,Slon
	# c:-10,150:2,9:50,1:23,59
	#
	# Note that the seconds should be kept as integers.  Items
	# stored within the cell can have decimal seconds, but the 
	# node itself should NOT have them.
	# 
	# Also cells can have null buckets for every bucket except the 10 degree (first)
	# and once a null bucket exists, *every* bucket afterwards MUST be null.
	#  example: c:-10,150:2,9:: is a valid cell name, but c:-10,150::50,1:23,59 is NOT!
	
	my $cell_name = "c:";

	$cell_name .= $self->{"cell"}->get_cell_name();
	$self->{'name'} = $cell_name;
      }
    else
      {
	# this is a generic node, we'll just return ourself
      }
	
    return $self->{'name'};
  }

sub set_name
  {
    my $self = shift;

    my ($name) = @_;

    return $self->{"name"} = $name;
  }

sub set_position
  {
    my $self = shift;
    
    # Use this to set the latitude/longitude we're talking about for the node.
    # This should only be used with type=cell

    return unless ($self->get_type() eq "cell");
    
    my ($cell_or_lat,$lon) = @_;

    my ($cell, $lat);

    if (ref $cell_or_lat)
      {
	# we're calling with a cell
	$cell = $cell_or_lat;
      }
    else
      {
	# we're calling with lat,lon
	$lat = $cell_or_lat;

	$cell = new LLI::Cell ( { 'lat' => $lat, 'lon' => $lon } );
      }

    $self->{'cell'} = $cell;
  }

sub cell_to_latlon
  {
    # this is just a helper function... probably shouldn't be here..
    my $self = shift;

    return $self->{'cell'}->cell_to_latlon(@_) if ($self->{'cell'});
  }

sub get_latlon_parts
  {
    # this returns an hashref with keys 10_lat, 1_lon, min_lat, sec_lon, (and their pairs)

    my $self = shift;

    return $self->{"cell"}->get_latlon_parts();
  }

sub get_type
  {
    my $self = shift;

    return $self->{'type'};
  }

sub set_type
  {
    my $self = shift;

    # This clears any previously set values and disambiguates what type
    # this node really is.  I guess these types should all have their
    # own object/class and then inherit from a generic node...but that
    # can be done later...right?

    my ($type) = @_;

    if ($type eq "cell")
      {
	$self->{'jid'} = undef;
      }
    elsif ($type eq "presence")
      {
	# undefine our cell data
	foreach my $key (keys %{ $self } )
	  {
	    $self->{$key} = undef if ($key =~ /\_(lat)|(lon)/);
	  }
      }
    else
      {
	# type = generic

	# undefine our cell data
        foreach my $key (keys %{ $self } )
          {
            $self->{$key} = undef if ($key =~ /\_(lat)|(lon)/);
          }

	$self->{'jid'} = undef;
      }

    $self->{'type'} = $type;

    return 1;
  }

sub set_resource
  {
    my $self = shift;

    my ($resource) = @_;
    
    $self->{"resource"} = $resource;
    return $self->{'resource'};
  }

sub get_resource
  {
    my $self = shift;
    
    return $self->{"resource"};
  }

sub set_jid
  {
    my $self = shift;

    my ($jid) = @_;
    
    $self->{"jid"} = $jid;
    $self->set_type("presence"); # set our type to presence
  }

sub get_jid
  {
    my $self = shift;
    
    return $self->{"jid"};
  }

sub pad
  {
    my $self = shift;

    # left pads a thing 050 from 50

    my ($string,$digits,$pad) = @_;

    $pad = $pad || "0";

    my $zeros = $pad x $digits; # 00000

    return substr($zeros.$string,-$digits,$digits);
  }

sub deliver_payloads
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'deliver_payloads'} = $set;
      }

    return $self->{'deliver_payloads'};
  }

sub persist_items
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'persist_items'} = $set;
      }

    return $self->{'persist_items'};
  }

sub expiration
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'expiration'} = $set;
      }

    # set a default 
    unless (defined $self->{'expiration'})
      {
	$self->{'expiration'} = 0; # never expire
      }

    return $self->{'expiration'};
  }

sub max_items
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'max_items'} = $set;
      }

    return $self->{'max_items'};
  }

sub owner
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'owner'} = $set;
      }

    if ($self->get_type() eq "presence")
      {
	$self->{'owner'} = $self->get_jid();
      }

    return $self->{'owner'};
  }

sub subscription
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'subscription'} = $set;
      }

    return $self->{'subscription'};
  }

sub publish
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'publish'} = $set;
      }

    return $self->{'publish'};
  }

sub max_payload_size
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'max_payload_size'} = $set;
      }

    return $self->{'max_payload_size'};
  }

sub send_last_with_subscribe
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'send_last_with_subscribe'} = $set;
      }

    return $self->{'send_last_with_subscribe'};
  }

sub id
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'nodeid'} = $set;
      }

    return $self->{'nodeid'};
  }

sub subscribers
  {
    my $self = shift;

    my ($set) = @_;

    if (defined $set)
      {
	$self->{'subscribers'} = $set; #should be an array ref
      }

    return $self->{'subscribers'};
  }

sub subscriber_add
  {
    my $self = shift;

    my ($jid) = @_;

    # get a list of subscribers
    my $subscribers = $self->subscribers();
    my (@subscribers,%subscribers);

    # this maybe should be done on the database which might do it faster?
    # we're just making sure that the subscriber list is unique

    # get the list of subscriber jids
    @subscribers = @{ $subscribers } if (ref $subscribers);
    
    # populate the hash table
    map { $subscribers{$_} = 1 } @subscribers;

    # add the current jid to the table
    $subscribers{$jid} = 1;

    # return the subscriber list back in array format
    @subscribers = keys %subscribers;

    # set this in our data structure
    return $self->subscribers(\@subscribers);
  }

sub subscriber_delete
  {
    my $self = shift;

    my ($jid) = @_;

    # get a list of subscribers
    my $subscribers = $self->subscribers();
    my (@subscribers,%subscribers);

    # this maybe should be done on the database which might do it faster?
    # we're just making sure that the subscriber list is unique

    # get the list of subscriber jids
    @subscribers = @{ $subscribers } if (ref $subscribers);
    
    # populate the hash table
    map { $subscribers{$_} = 1 } @subscribers;

    # delete the current jid 
    undef $subscribers{$jid};

    # return the subscriber list back in array format
    @subscribers = keys %subscribers;

    # set this in our data structure
    return $self->subscribers(\@subscribers);
  }

sub add_nugget_part
  {
    # this takes a bit of information (maybe from a database) and then 
    my $self = shift;

    my ($data) = @_; # hashref with all the nugget data

    
  }

1;
